% The codes load and display the signals from the St-Vincents database with
% groundtruth apnea-hypopnea events
dbstop if error
% loading sleep signals and the notations
addpath('../edfread');
fileDir = '../../Data/st-vincents-university-hospital-university-college-dublin-sleep-apnea-database-1.0.0/';
fileName = 'ucddb002';

channelName = {'SpO2';'Flow'};
name4Disp = {'SpO2';'Oronasal airflow'};
epoch = 30; % epoch length in s

% ************************** plot setting *******************************
plotClrApnea = [7 3 3]/8;
plotClrHPY = [7 7 7]/8;
wdSzInSec = 180; % window size in seconds
burnin_Num = 10; % the number of epoches which won't be processed at the beginning
% loading data and ground truth
% sleep stage: 0 - Wake, 1 - REM, 2 - Stage 1, 3 - Stage 2, 4 - Stage 3, 
% 5 - Stage 4, 6 - Artifact, 7 - Indeterminate0
% slpStg_gt = importdata(fullfile(fileDir, strcat(fileName,'_stage.txt')));

%**************** loading the respiratory events***************************
respEvt_gt = importdata(fullfile(fileDir, strcat(fileName,'_respevt.txt')));
respEvt_gt = respEvt_gt(4:end); % remove the title lines
arrLen = num2cell(3*ones(size(respEvt_gt,1),1));
tmp = cellfun(@splitIneqStr,respEvt_gt,arrLen,'UniformOutput',false);
respEvt_gt = tmp(~cellfun(@isempty,tmp)); % respitory event ground truth
respEvt_num = length(respEvt_gt);

% ************loading and extracting the target signals *******************
[hdr, record] = edfread(fullfile(fileDir,strcat(fileName,'.rec')));

channelNum = length(channelName);
tg_Sig = cell(channelNum,1);
seqLen = zeros(channelNum,1);
smp_Rt = zeros(channelNum,1);
yRg = cell(channelNum,1);
for i = 1:channelNum
    nameCell = cell(1,length(hdr.label));
    [nameCell{1:end}] = deal(channelName{i});
    channelInd = find(cellfun(@strcmp,hdr.label,nameCell)); % Signal index

    smp_Rt(i) = hdr.frequency(channelInd); %sampling frequency, data points per second
    seqLen(i) = floor(size(record,2)*smp_Rt(i)/max(hdr.frequency));
    tg_Sig{i} = record(channelInd,1:seqLen(i))'; % target data
    yRg{i} = FindDispRg(tg_Sig{i}, burnin_Num*epoch*smp_Rt(i)); % set the disp axis range
end
clear record;

starttime = split(hdr.starttime,'.'); % starting of the records

% % gt by segments
% segTime = 60; % the time(in second) of each segment which the signal is divided into
% subSeqLen = segTime*smp_Rt; % segment length (data point number)
% segGt = zeros(ceil(seqLen/subSeqLen),1); % groundtruth per segment

% indGt = zeros(seqLen,1);
% for i = 1:respEvt_num
%     gtStart = split(respEvt_gt{i}{1},':');
%     gtStartInd = timeDif(starttime,gtStart)*smp_Rt+1;
%     gtEnd = gtStartInd+str2double(respEvt_gt{i}{3})*smp_Rt-1;
%     if respEvt_gt{i}{2}(1)=='H'
%         indGt(gtStartInd:gtEnd) = 1;
%     elseif respEvt_gt{i}{2}(1)=='A'
%         indGt(gtStartInd:gtEnd) = 2;
%     end
%     segGt(ceil(gtStartInd/subSeqLen):ceil(gtEnd/subSeqLen))...
%                                     = max(indGt(gtStartInd:gtEnd));
% end


% ********************* plotting the target data **************************
wdSz = wdSzInSec*smp_Rt; % plot window size 
respGtPt = 1;
wdNum = ceil(seqLen(1)/wdSz(1));
figure(1);
for winInd = 1:wdNum
    startInd = (winInd-1)*wdSz+1;
    endInd = min(seqLen,startInd+wdSz-1);
    if respGtPt<=respEvt_num
        gtStart = split(respEvt_gt{respGtPt}{1},':');
        gtStartInd = timeDif(starttime,gtStart)*smp_Rt+1;
        gtEnd = gtStartInd+str2double(respEvt_gt{respGtPt}{3})*smp_Rt-1;
    end

    while endInd(1)>=gtEnd(1)&&respGtPt<=respEvt_num
        if startInd<=gtStartInd
            for sigInd = 1:channelNum
                subplot(channelNum,1,sigInd); 
                x = [gtStartInd(sigInd):gtEnd(sigInd)]';
                y = yRg{sigInd}.high*ones(gtEnd(sigInd)-gtStartInd(sigInd)+1,1);
                x = [x;flipdim(x,1)];
                y = [y;flipdim(yRg{sigInd}.low*ones(gtEnd(sigInd)-gtStartInd(sigInd)+1,1),1)];
                if respEvt_gt{respGtPt}{2}(1)=='H'
                    fill(x,y,plotClrHPY); 
                elseif respEvt_gt{respGtPt}{2}(1)=='A'
                    fill(x,y,plotClrApnea);
                end
                % set the legend of colour filling part
                [~,h_legend] = legend('Apnea-hypopnea','FontSize',24);
                PatchInLegend = findobj(h_legend, 'type', 'patch');
                set(PatchInLegend(1), 'FaceAlpha', 1);
                hold on;
            end
        elseif gtEnd(1)>=startInd(1)&&startInd(1)>gtStartInd(1)
            for sigInd = 1:channelNum
                subplot(channelNum,1,sigInd);
                x = [startInd(sigInd):gtEnd(sigInd)]';
                y = yRg{sigInd}.high*ones(gtEnd(sigInd)-startInd(sigInd)+1,1);
                x = [x;flipdim(x,1)]; 
                y = [y;flipdim(yRg{sigInd}.low*ones(gtEnd(sigInd)-startInd(sigInd)+1,1),1)];
                if respEvt_gt{respGtPt}{2}(1)=='H'
                    fill(x,y,plotClrHPY); 
                elseif respEvt_gt{respGtPt}{2}(1)=='A'
                    fill(x,y,plotClrApnea);
                end 
                % set the legend of colour filling part
                [~,h_legend] = legend('Apnea-hypopnea','FontSize',24);
                PatchInLegend = findobj(h_legend, 'type', 'patch');
                set(PatchInLegend(1), 'FaceAlpha', 1);
                hold on;
            end
        end
        
        if respGtPt>respEvt_num
            break;
        end
        respGtPt = respGtPt+1;
        if respGtPt<=respEvt_num
            gtStart = split(respEvt_gt{respGtPt}{1},':');
            gtStartInd = timeDif(starttime,gtStart)*smp_Rt+1;
            gtEnd = gtStartInd+str2double(respEvt_gt{respGtPt}{3})*smp_Rt-1;
        end
    end
    
    if gtStartInd(1)<=endInd(1)&&startInd(1)<=gtStartInd(1)&&endInd(1)<gtEnd(1)
        for sigInd = 1:channelNum
            subplot(channelNum,1,sigInd);
            x = [gtStartInd(sigInd):endInd(sigInd)]';
            y = yRg{sigInd}.high*ones(endInd(sigInd)-gtStartInd(sigInd)+1,1);
            x = [x;flipdim(x,1)]; 
            y = [y;flipdim(yRg{sigInd}.low*ones(endInd(sigInd)-gtStartInd(sigInd)+1,1),1)];
            if respEvt_gt{respGtPt}{2}(1)=='H'
                fill(x,y,plotClrHPY); 
            elseif respEvt_gt{respGtPt}{2}(1)=='A'
                fill(x,y,plotClrApnea);
            end 
            % set the legend of colour filling part
            [~,h_legend] = legend('Apnea-hypopnea','FontSize',24);
            PatchInLegend = findobj(h_legend, 'type', 'patch');
            set(PatchInLegend(1), 'FaceAlpha', 1);
            hold on;
        end
%         fprintf('First half Time:%s, Type:%s\n', respEvt_gt{respGtPt}{1},respEvt_gt{respGtPt}{2});
    end
    for sigInd = 1:channelNum
        subplot(channelNum,1,sigInd);
        h = plot(startInd(sigInd):endInd(sigInd), tg_Sig{sigInd}(startInd(sigInd):endInd(sigInd)),...
            'LineWidth',2, 'Color','b'); hold on;
        ylim([yRg{sigInd}.low,yRg{sigInd}.high]); xlim([startInd(sigInd),endInd(sigInd)]);
        set(gca, 'XTick', [], 'FontSize',24); 
        title(channelName{sigInd}); 
        xline(floor((endInd(sigInd)+2*startInd(sigInd))/3),'--p','LineWidth', 0.8);
        xline(floor((2*endInd(sigInd)+startInd(sigInd))/3),'--p','LineWidth', 0.8);
        hold off;
%     xticks(spInd:subSeqLen:endInd);   
    end
    if min(tg_Sig{2}(startInd(2):endInd(2)))<50
        disp('stop');
    end
    if winInd == 99
        disp('pause');
    end
%     pause(2);
end
% export_fig Apnea_3Signals_1.eps
disp('End');

% for ind = 1:100:num
%     fprintf('From %d to %d, mean is %f\n', ind, ind+100-1, mean(spo2(ind:ind+100-1)))
% end
