%% Extract features from sub-sequences of signals and save them into a file

dbstop if error
% loading sleep signals and the notations
pathConfig();
dataSet = 'ECG_DB';
if strcmp(dataSet, 'St_Vincents')
    fileDir = '../../Data/st-vincents-university-hospital-university-college-dublin-sleep-apnea-database-1.0.0/';
elseif strcmp(dataSet, 'ECG_DB')
    fileDir = '../../Data/apnea_ecg_database/';
end
files = dir(fileDir);

% ********************** Parameter setting *****************************
channelName = 'SpO2';
epoch = 30; % epoch length in seconds
segStep = 60; % the step of the beginning of each segment (in seconds)
segTime = 60; % the time (in second) of each segment which the signal is divided into
% the segments are overlapped if segTime > segStep
apTh = 5; % apnea length in a segment to define it apnea
afTh = 50; % artifact threshold

featNames = {'CTM','LZC','t_M4','t_M2','SAMPEN','Rg','MinV','Power_Wavdb1D',...
            'Wav_db1_L2Num','Wav_db1_L3Num','Wav_db1_L4Num','Wav_db1_L5Num',...
            'Wav_db1L6_max','Wav_db1L7_max','Entropy_Wavdb1D', 'Var_Wavdb1D'};
% featNames = {'Power_Wavdb1D'};

featNamesAll = FeatNameTrans(featNames); % transfer the group feat names to seperate ones
featNum = length(featNamesAll);
%%
for fileInd = 1:length(files)
    fileName = files(fileInd).name;
%     if files(fileInd).isdir||~strcmp(fileName(end-3:end),'.rec')
% %         ||strcmp(fileName,'ucddb006.rec')||strcmp(fileName,'ucddb027.rec')
%         continue;
%     end
    
    if files(fileInd).isdir||~strcmp(fileName(max(1,end-4):end),'r.dat')
        continue;
    end
    fprintf('%s\n',fileName);
%     if ~strcmp(fileName,'ucddb006.rec')
%         continue;
%     end

[tgSig,segGt,subSeqStep] = sigLoading(fileDir, fileName, channelName, ...
                                           segStep, segTime-segStep, apTh);
subSeqLen = subSeqStep*segTime/segStep;
%% ************************** Feature Extraction *************************
    maxDif = max(tgSig(tgSig>afTh))-min(tgSig(tgSig>afTh)); % the range for nomalization
    seqLen = max(size(tgSig)); % sequence length
    featStruct = [];
    featStruct.featNames = featNamesAll;
    feat = zeros(length(segGt),featNum);
    tmpPt = 1;
    for sigInd = 1:subSeqStep:seqLen-subSeqLen+1
        curSig = tgSig(sigInd:min(sigInd+subSeqLen-1,seqLen));
        curSig = downsample(curSig, 10); % downsample the signals from ECG-DB
        if any(curSig<afTh)
            feat(tmpPt,:) = NaN; 
            tmpPt = tmpPt+1;
            continue;
        end
        input.sig = curSig;
%         input.maxDif = maxDif;
       
%         disp(segGt(tmpPt));
        featPt = 1; % to deal with the wavelet detail coefficient features
        for featInd = 1:length(featNames)
            input.featName = featNames{featInd};
            tmpFeat = featExt(input);
            feat(tmpPt,featPt:featPt+length(tmpFeat)-1) = tmpFeat;
            featPt = featPt+length(tmpFeat);
        end
        tmpPt = tmpPt+1;
    end
    
    featStruct.segGt = segGt;
    featStruct.feat = feat;

%     fprintf('gt:0, mean:%f, std:%f\n',mean(feat(bsxfun(@and,segGt==0, ~isnan(feat)))),...
%         std(feat(bsxfun(@and,segGt==0, ~isnan(feat)))));
%     fprintf('gt:2||3, mean:%f, std:%f\n',mean(feat(bsxfun(@and,segGt>1, ~isnan(feat)))),...
%         std(feat(bsxfun(@and,segGt>2, ~isnan(feat)))));
    
    save(fullfile('../Features4HealthySleep/Wavelet/AllFeatPro_novlp_ds/',dataSet, strcat(fileName(1:end-4),'_',channelName,...
        '_', num2str(segStep), '.mat')),'featStruct');
end

%************************ Display features *******************************    
% figure(1);h1 = histogram(featAll(bsxfun(@and,featAll(:,2)==0, ~isnan(featAll(:,1))),1),'FaceColor','r'); 
% h1.Normalization = 'probability';%h1.BinWidth = 0.001; 
% hold on;
% h2 = histogram(featAll(bsxfun(@and,featAll(:,2)>1, ~isnan(featAll(:,1))),1),'FaceColor','b');
% h2.Normalization = 'probability';%h2.BinWidth = 0.001;
% legend('Normal','Apnea'); hold off;
