%% Codes for extracting wavlet based features from sub-signals, these codes
% designed for choosing proper threshods of wavelet detail coefficients
% from ROC curves
dbstop if error
% loading sleep signals and the notations
pathConfig();
dataSet = 'St_Vincents';
if strcmp(dataSet, 'St_Vincents')
    fileDir = '../../Data/st-vincents-university-hospital-university-college-dublin-sleep-apnea-database-1.0.0/';
elseif strcmp(dataSet, 'ECG_DB')
    fileDir = '../../Data/apnea_ecg_database/';
end
files = dir(fileDir);

% ********************** Parameter setting *****************************
channelName = 'SpO2';
epoch = 30; % epoch length in seconds
segTime = 60; % the time(in second) of each segment which the signal is divided into
apTh = 5; % apnea length in a segment to define it apnea
afTh = 50; % artifact threshold
wavTh = [0.4 0.5 0.55 0.6 0.7;
         0.5 0.6 0.65 0.7 0.8];
thNum = size(wavTh,2);

% featNames = {'CTM','LZC','t_M4','t_M2','SAMPEN','DI'};
featNames = {'Wav_db1_L2','Wav_db1_L3'};
%% wavelet detail coefficient energy as features
% featNames = {'Wav_db1_D'};

tmp = cell(size(featNames));
tmp(:) = {'Wav_db1_D'};
if any(cellfun(@strcmp,featNames,tmp))
    featNum = length(featNames)+6;
else
    featNum = length(featNames);
end
clear tmp;
%%
for fileInd = 3:length(files)
    fileName = files(fileInd).name;
    if files(fileInd).isdir||~strcmp(fileName(end-3:end),'.rec')
%         ||strcmp(fileName,'ucddb006.rec')||strcmp(fileName,'ucddb027.rec')
        continue;
    end
    
%     if files(fileInd).isdir||~strcmp(fileName(max(1,end-4):end),'r.dat')
%         continue;
%     end
    fprintf('%s\n',fileName);
%     if ~strcmp(fileName,'ucddb006.rec')
%         continue;
%     end

[tgSig,segGt,subSeqLen] = sigLoading(fileDir, fileName, channelName, segTime, apTh);

%% ************************** Feature Extraction *************************
    maxDif = max(tgSig(tgSig>afTh))-min(tgSig(tgSig>afTh)); % the range for nomalization
    seqLen = max(size(tgSig));
    featStruct = [];
    featStruct.featNames = featNames;
    feat = zeros(length(segGt),featNum,thNum);
    tmpPt = 1;
    for sigInd = 1:subSeqLen:seqLen-subSeqLen+1
        curSig = tgSig(sigInd:min(sigInd+subSeqLen-1,seqLen));
        if any(curSig<afTh)
            feat(tmpPt,:) = NaN; 
            tmpPt = tmpPt+1;
            continue;
        end
        input.sig = curSig;
        input.maxDif = maxDif;
        
        for featInd = 1:featNum
            input.featName = featNames{featInd};
            input.ths = wavTh(featInd,:);
            feat(tmpPt,featInd,:) = featExt(input);
        end
        
%         input.featName = featNames{:};
%         feat(tmpPt,:) = featExt(input);

        tmpPt = tmpPt+1;
    end
    featStruct.segGt = segGt;
    featStruct.feat = feat;


%     fprintf('gt:0, mean:%f, std:%f\n',mean(feat(bsxfun(@and,segGt==0, ~isnan(feat)))),...
%         std(feat(bsxfun(@and,segGt==0, ~isnan(feat)))));
%     fprintf('gt:2||3, mean:%f, std:%f\n',mean(feat(bsxfun(@and,segGt>1, ~isnan(feat)))),...
%         std(feat(bsxfun(@and,segGt>2, ~isnan(feat)))));
    
    save(fullfile('../Features4HealthySleep/Wavelet/db1_L23_num/',dataSet, strcat(fileName(1:end-4),'_',channelName,...
        '_', num2str(segTime), '.mat')),'featStruct');
end

%************************ Display features *******************************    
% figure(1);h1 = histogram(featAll(bsxfun(@and,featAll(:,2)==0, ~isnan(featAll(:,1))),1),'FaceColor','r'); 
% h1.Normalization = 'probability';%h1.BinWidth = 0.001; 
% hold on;
% h2 = histogram(featAll(bsxfun(@and,featAll(:,2)>1, ~isnan(featAll(:,1))),1),'FaceColor','b');
% h2.Normalization = 'probability';%h2.BinWidth = 0.001;
% legend('Normal','Apnea'); hold off;
