% The codes perform the k-folds validation with the DPGMM model on all 
% possbile subsets of features
% **********************************************************************
% dataSet = 'St_Vincents';
dataSet = 'St_Vincents';
fileDir = fullfile('../Features4HealthySleep/Wavelet/WavNumPro_ovlp_15/',dataSet);
if strcmp(dataSet,'St_Vincents')
    respLv = 3; % problems equal and worse will be considered: 1--possible, 3--hyp, 4--apnea
elseif strcmp(dataSet,'ECG_DB')
    respLv = 1;
end
% trainPropt = 0.7;

featAll = [];
gtAll = [];
files = dir(fileDir);
for fileInd = 1:length(files)
    if files(fileInd).isdir
        continue;
    end
    fileName = files(fileInd).name;
    load(fullfile(fileDir,fileName),'featStruct');
    featAll = cat(1,featAll,featStruct.feat);
    gtAll = cat(1,gtAll,featStruct.segGt);
end

allFeats = featStruct.featNames;
selectFeats = {'CTM','t_M2','MinV','Rg','Wav_db1_L3Num','Wav_db1_L2Num',...
    'Wav_db1_L4Num','Wav_db1_L5Num','Entropy_Wavdb1D_2', 'Var_Wavdb1D_4','Wav_db1L6_max'}; 

selectInd = arrayfun(@(j)find(arrayfun(@(i)strcmp(selectFeats{j}, allFeats{i}),...
    1:length(allFeats))),1:length(selectFeats));

ind = ff2n(length(selectFeats));
ind = ind(2:end,:);
featSubset = arrayfun(@(i)selectInd(logical(ind(i,:))),1:size(ind,1),'un',0)';
clear tmp;clear ind;

subsetNum = length(featSubset);
perfStruct = struct('feats',cell(subsetNum,1),'accuracy', [], ...
                                    'sensitivity', [], 'specificity', []);

for subsetInd = 1:subsetNum
    prbFeat = featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,2))),featSubset{subsetInd}); % attention for NaN check
    normalFeat = featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,2))),featSubset{subsetInd});
    prbNum = size(prbFeat,1);
    normalNum = size(normalFeat,1);
    gtLabel = cat(1,true(prbNum,1),false(normalNum,1));

    % cross validation with k-folds
    seed = 100; s = rng(seed,'twister'); % make sure each feat are trained and the tested on the same partitions
    cp = cvpartition(gtLabel,'k',10); % Stratified cross-validation
    order = unique(gtLabel);

    f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,...
        DP_Classify(xTrain, yTrain, xTest, 'multiVari'),'order',order);

    cfMat = crossval(f,cat(1,prbFeat,normalFeat),gtLabel,'partition',cp);
    cfMat = reshape(sum(cfMat),2,2);
    % confusionchart(cfMat,order)
    sens = cfMat(order==true,order==true)/sum(cfMat(order==true,:));
    spec = cfMat(order==false,order==false)/sum(cfMat(order==false,:));
    acc = trace(cfMat)/sum(cfMat,'All');
    fprintf('10-folds Accuracy:%f, Sensitivity:%f, Specificity:%f\n',acc, sens, spec);
    perfStruct(subsetInd).feats = {featStruct.featNames{featSubset{subsetInd}}};
%     perfStruct(subsetInd).feats = featSubset{subsetInd}; % for wavelet detail coef energy as feats
    perfStruct(subsetInd).accuracy = acc;
    perfStruct(subsetInd).sensitivity = sens;
    perfStruct(subsetInd).specificity = spec;
end
save(strcat('Results_',dataSet,'_fullset_ovlp15.mat'),'perfStruct');
