% The codes collect data from files containing the features extracted
% from different signals and perform the k-folds validation with the DPGMM models
% It performs a forward feat selection to pick features providing best
% performance
% **********************************************************************
% dataSet = 'St_Vincents';
dataSet = 'St_Vincents';
fileDir = fullfile('../Features4HealthySleep/Wavelet/WavNumPro_ovlp_15/',dataSet);
if strcmp(dataSet,'St_Vincents')
    respLv = 3; % problems equal and worse will be considered: 1--possible, 3--hyp, 4--apnea
elseif strcmp(dataSet,'ECG_DB')
    respLv = 1;
end

featAll = [];
gtAll = [];
files = dir(fileDir);
for fileInd = 1:length(files)
    if files(fileInd).isdir
        continue;
    end
    fileName = files(fileInd).name;
    load(fullfile(fileDir,fileName),'featStruct');
    featAll = cat(1,featAll,featStruct.feat);
    gtAll = cat(1,gtAll,featStruct.segGt);
end

featAll = featAll(:,cat(2,1:19,21:length(featStruct.featNames))); % choose features 
featNames = featStruct.featNames;
featNames = featNames(cat(2,1:19,21:length(featStruct.featNames)));


% ind = ff2n(length(featStruct.featNames));
% % ind = ff2n(7);
% ind = ind(2:end,:);
% tmp = 1:length(featStruct.featNames);
% % tmp = 1:7;
% featSubset = arrayfun(@(i)tmp(logical(ind(i,:))),1:size(ind,1),'un',0)';
% clear tmp;clear ind;
featNum = length(featNames);

perfStruct = struct('feats',cell(featNum,1),'accuracy', [], ...
                                    'sensitivity', [], 'specificity', []);
selectFeat = [];
for selectFeatNum = 0:featNum-1
    feats = 1:featNum;
    featLeft = feats;
    for i = 1:length(selectFeat)
        featLeft(feats==selectFeat(i)) = 0;
    end
    featLeft = featLeft(featLeft~=0);
    featSubset = arrayfun(@(j)cat(2,selectFeat,featLeft(j)), ... 
            (1:length(featLeft))','UniformOutput',0); % subset of feats to be modeled
    
    % struct arrary of results with diff features for feat selection
    perf4FeatSelect = struct('feats',featSubset,'accuracy', [], ...
                                    'sensitivity', [], 'specificity', []);
    for subsetInd = 1:featNum-selectFeatNum
        prbFeat = featAll(bsxfun(@and,gtAll>=respLv, ~isnan(featAll(:,3))),featSubset{subsetInd}); % attention for NaN check
        normalFeat = featAll(bsxfun(@and,gtAll==0, ~isnan(featAll(:,3))),featSubset{subsetInd});
        prbNum = size(prbFeat,1);
        normalNum = size(normalFeat,1);
        gtLabel = cat(1,true(prbNum,1),false(normalNum,1));

        % cross validation with k-folds
        seed = 100; s = rng(seed,'twister'); % make sure each feat are trained and the tested on the same partitions
        cp = cvpartition(gtLabel,'k',10); % Stratified cross-validation
        order = unique(gtLabel);

        f = @(xTrain, yTrain, xTest,yTest)confusionmat(yTest,DP_Classify(xTrain, yTrain, xTest, 'multiVari'),'order',order);

        cfMat = crossval(f,cat(1,prbFeat,normalFeat),gtLabel,'partition',cp);
        cfMat = reshape(sum(cfMat),2,2);
        % confusionchart(cfMat,order)
        sens = cfMat(order==true,order==true)/sum(cfMat(order==true,:));
        spec = cfMat(order==false,order==false)/sum(cfMat(order==false,:));
        acc = trace(cfMat)/sum(cfMat,'All');
        fprintf('10-folds Accuracy:%f, Sensitivity:%f, Specificity:%f\n',acc, sens, spec);
        perf4FeatSelect(subsetInd).feats = {featNames{featSubset{subsetInd}}};
        perf4FeatSelect(subsetInd).accuracy = acc;
        perf4FeatSelect(subsetInd).sensitivity = sens;
        perf4FeatSelect(subsetInd).specificity = spec;
    end
%     [~,bestSubsetInd] = max(cell2mat({perf4FeatSelect.accuracy})...
%         +cell2mat({perf4FeatSelect.sensitivity}));% choose the feats have highest acc+sens
      [~,bestSubsetInd] = max(cell2mat({perf4FeatSelect.accuracy}));
    selectFeat = featSubset{bestSubsetInd};
    perfStruct(selectFeatNum+1) = perf4FeatSelect(bestSubsetInd);
end
save(strcat('Results_',dataSet,'_WavNumPro_ovlp15_fwdFeatSelect.mat'),'perfStruct');
