function feat = featExt(input)
% input: a struct 
% input.sig: input signals for processing
% input.featName: the name of features to be extracted
% featName list:
% Lempel-Ziv compexity (LZC)
% Central Tendency measure (CTM)
% Delta index (DI)
% Sample entropy (SAMPEN)
% output:
% feat: extracted features
%%***********************************************************************
if strcmp(input.featName,'LZC') % Lempel-Ziv compexity (LZC)
    biSig = true(size(input.sig));
    biSig((input.sig-mean(input.sig))<0)=false;
    [feat,~,~] = calc_lz_complexity(biSig,'primitive',1);
end

if strcmp(input.featName,'CTM') % Central Tendency measure (CTM)
    feat = calc_CTM(input.sig, 1);
end

% Relative power: ratio of apnea related power to total spectral power
if strcmp(input.featName, 'Pr')
    curSig = input.sig-mean(input.sig);
    Nx = length(input.sig);
    nsc = floor(Nx/4.5);
    nov = floor(nsc/2);
    nff = max(2^13,2^nextpow2(nsc));
    [pxx,w] = pwelch(curSig,hamming(nsc),nov,nff);
    [~,indl] = min(abs(w-0.014));
    [~,indh] = min(abs(w-0.033));
    feat = sum(pxx(indl:indh))/sum(pxx);
end

if strcmp(input.featName,'t_M2') % variance
%     feat = var(input.sig/input.maxDif);
    feat = var(input.sig);
end

if strcmp(input.featName,'t_M4')
    feat = kurtosis(input.sig);
end

if strcmp(input.featName, 'DI')
    inputShift1 = circshift(input.sig,1);
    dif = abs(input.sig-inputShift1);
    feat = mean(dif(2:end));
end

if strcmp(input.featName, 'SAMPEN') % sample entropy
    [feat,~,~,~]=sampen(input.sig,1,0.25,1,0);
end


if strcmp(input.featName, 'Power_Wavdb1D')
    [c,l] = wavedec(input.sig,7,'db1');
    feat = zeros(1,7);
%     figure(1);
    for i = 1:7
        c_d = detcoef(c,l,i);
        feat(i) = mean(c_d.*c_d);       
%         subplot(4,2,i+1); plot(c_d); ylim([-2.5*2^floor(i/3), 2.5*2^floor(i/3)]);
%         xlim([0,length(detcoef(c,l,i))]);
%         xlabel('Index number'); ylabel(sprintf('L%d wavelet Coefficients\n',i));
%         title(sprintf('Level %d Detail Coefficients\n',i));        
    end
%     subplot(4,2,1); plot(input.sig);  ylim([88,100]);
%     title(sprintf('Input signal:\n'));
    % codes for saving plots
%     set(gcf, 'Position',  [100, 100, 500, 200])
%     print -depsc2 'apneaL2.eps'
%         pause(10); 
end

if strcmp(input.featName, 'Entropy_Wavdb1D')
    [c,l] = wavedec(input.sig,7,'db1');
    feat = zeros(1,7);
%     figure(1);
    for i = 1:7
        c_d = detcoef(c,l,i);
        [feat(i),~,~,~] = sampen(c_d,1,0.25,1,0);   
    end
end

if strcmp(input.featName, 'Var_Wavdb1D')
    [c,l] = wavedec(input.sig,7,'db1');
    feat = zeros(1,7);
%     figure(1);
    for i = 1:7
        c_d = detcoef(c,l,i);
        feat(i) = var(c_d);   
    end
end

% number of points larger than a th of the level 2 detailed coefficients of
% db1(Haar) wavelet transform
if strcmp(input.featName, 'Wav_db1_L2Num')
%     ths = input.ths;
%     thNum = length(ths);
    [c,l] = wavedec(input.sig,2,'db1');
    c_d = detcoef(c,l,2);
%     feat = arrayfun(@(i)sum(abs(c_d)>ths(i)), 1:thNum);
    feat = sum(abs(c_d)>0.4);
end

% number of points larger than a th of the level 3 detailed coefficients of
% db1(Haar) wavelet transform
if strcmp(input.featName, 'Wav_db1_L3Num')
    [c,l] = wavedec(input.sig,3,'db1');
    c_d = detcoef(c,l,3);
    feat = sum(abs(c_d)>0.7);
end

if strcmp(input.featName, 'Wav_db1_L4Num')
    [c,l] = wavedec(input.sig,4,'db1');
    c_d = detcoef(c,l,4);
    feat = sum(abs(c_d)>2.5);
end

if strcmp(input.featName, 'Wav_db1_L5Num')
    [c,l] = wavedec(input.sig,5,'db1');
    c_d = detcoef(c,l,5);
    feat = sum(abs(c_d)>3.8);
end

if strcmp(input.featName, 'Wav_db1L7_max')
    [c,l] = wavedec(input.sig,7,'db1');
    c_d = detcoef(c,l,7);
    feat = max(abs(c_d));
end

if strcmp(input.featName, 'Wav_db1L6_max')
    [c,l] = wavedec(input.sig,6,'db1');
    c_d = detcoef(c,l,6);
    feat = max(abs(c_d));
end

% max - min
if strcmp(input.featName, 'Rg')
   feat = max(input.sig)-min(input.sig);
end

% min value
if strcmp(input.featName, 'MinV')
    feat = min(input.sig);
end

% crossing zero point number
if strcmp(input.featName, 'ZeroCross')
    sig = input.sig-mean(input.sig);
    tmp = (sig(:).*circshift(sig(:), [-1 0])<=0);
%     zci = @(v) find(v(:).*circshift(v(:), [-1 0]) <= 0);
    feat = length(find(tmp(1:end-1)));
end
end