function [tgSig,segGt,subSeqLen] = sigLoading(fileDir,fileName, channelName, segTime, ovlp, apTh)
% loading target signals and load/transfer groundtruth into segment based ones
% input:
% fileDir: the dir of files
% fileName : file containing signals
% channelName: the name of the target feature to be processed
% segTime: the time(in second) of each segment which the signal is divided into
% apTh: apnea length in a segment to define it apnea
% output:
% tgSig: target signal
% subSeqLen: segment length (data point number)
% segGt: groundtruth per segment
%% ***********************************************************************
addpath('/home/zhenglin/Documents/Research/HealthySleep/Codes/wfdb_app_toolbox');
pathSeg = split(fileDir,'/');

if strcmp(pathSeg{end-1},'st-vincents-university-hospital-university-college-dublin-sleep-apnea-database-1.0.0')   
    %% **************** loading the respiratory events*************************
    respEvt_gt = importdata(fullfile(fileDir, strcat(fileName(1:end-4),'_respevt.txt')));
    respEvt_gt = respEvt_gt(4:end); % remove the title lines
    arrLen = num2cell(3*ones(size(respEvt_gt,1),1));
    tmp = cellfun(@splitIneqStr,respEvt_gt,arrLen,'UniformOutput',false);
    respEvt_gt = tmp(~cellfun(@isempty,tmp)); % respitory event ground truth
    respEvt_num = length(respEvt_gt);
%**************************************************************************

%% ******************** loading the target signals ************************
    [hdr, record] = edfread(fullfile(fileDir,fileName));
    nameCell = cell(1,length(hdr.label));
    [nameCell{1:end}] = deal(channelName);
    channelInd = find(cellfun(@strcmp,hdr.label,nameCell)); % Signal index
    smpRt = hdr.frequency(channelInd); %sampling frequency, data points per second
    seqLen = floor(size(record,2)*smpRt/max(hdr.frequency));
    tgSig = record(channelInd,1:seqLen)'; % target data
    clear record;
    starttime = split(hdr.starttime,'.'); % starting of the records

%% ******************* Transfer resp gt to segment-wise *******************
    subSeqLen = segTime*smpRt; % segment length (data point number)
    ovlpLen = ovlp*smpRt; % overlap length
    segGt = zeros(floor((seqLen-ovlpLen-subSeqLen)/subSeqLen)+1,1); % groundtruth per segment
    tgSig = tgSig(1:length(segGt)*subSeqLen+ovlpLen);
    indGt = zeros(seqLen,1); % data point-wise groundtruth
    % segSig = zeros(floor(seqLen/subSeqLen),subSeqLen);
    
    for i = 1:respEvt_num
        gtStart = split(respEvt_gt{i}{1},':');
        gtStartInd = timeDif(starttime,gtStart)*smpRt+1;
        gtEndInd = gtStartInd+str2double(respEvt_gt{i}{3})*smpRt-1;
        if respEvt_gt{i}{2}(1)=='H'
            indGt(gtStartInd:gtEndInd) = 3;
        elseif respEvt_gt{i}{2}(1)=='A'
            indGt(gtStartInd:gtEndInd) = 4;
        elseif strcmp(respEvt_gt{i}{2}(1:2),'PO')
            indGt(gtStartInd:gtEndInd) = 1;
        elseif strcmp(respEvt_gt{i}{2}(1:2),'PB')
            indGt(gtStartInd:gtEndInd) = 2;
        end
%         segGt(ceil(gtStartInd/subSeqLen):ceil(gtEnd/subSeqLen))...
%             = max(indGt(gtStartInd:gtEnd)); % turn index gt to segment gt

        % turn index gt to segment gt
        if ceil(gtStartInd/subSeqLen)==ceil(gtEndInd/subSeqLen) 
            % the apnea event happens inside a segment (gtStartInd and gtEndInd in the same seg)
            segGt(ceil(gtStartInd/subSeqLen)) = ...
                max(segGt(ceil(gtStartInd/subSeqLen)),max(indGt(gtStartInd:gtEndInd)));

        else % the apnea event cross two segments (gtStartInd and gtEndInd in two consecutive segments)
            if (ceil(gtStartInd/subSeqLen)*subSeqLen-gtStartInd+1)>=apTh*smpRt % apnea in the former seg longer than apTh seconds as apnea
                segGt(ceil(gtStartInd/subSeqLen)) = max(segGt(ceil(gtStartInd/subSeqLen)),max(indGt(gtStartInd:gtEndInd)));
            end
            if (gtEndInd-floor(gtEndInd/subSeqLen)*subSeqLen)>=apTh*smpRt % apnea in the latter seg longer than apTh seconds as apnea
                segGt(ceil(gtEndInd/subSeqLen)) = max(segGt(ceil(gtEndInd/subSeqLen)),max(indGt(gtStartInd:gtEndInd)));
            end

%             % apnea in the latter seg as apnea (no matter how long it is)
%             segGt(ceil(gtEndInd/subSeqLen)) = max(segGt(ceil(gtEndInd/subSeqLen)),max(indGt(gtStartInd:gtEndInd))); 
        end
    end
elseif strcmp(pathSeg{end-1},'apnea_ecg_database')
    curDir = pwd;
    cd(fileDir);
    [tgSig,smpRt,tm]=rdsamp(fileName(1:end-4));
    [ann,segGt,~,~,~]=rdann(fileName(1:end-4), 'apn'); % loading the groundtruth
    ovlpLen = ovlp*smpRt;
    cd(curDir);
    subSeqLen = ann(2)-ann(1);
    if strcmp(channelName, 'SpO2')
        channelInd = 4;
    end
    if (ann(end)+subSeqLen+ovlpLen-1)>size(tgSig,1)&& (ann(end-1)+subSeqLen+ovlpLen-1)>size(tgSig,1)
        tgSig = tgSig(ann(1):ann(end-2)+subSeqLen+ovlpLen-1,channelInd);
        segGt = segGt(1:end-2);
    elseif (ann(end)+subSeqLen+ovlpLen-1)>size(tgSig,1)&& (ann(end-1)+subSeqLen+ovlpLen-1)<=size(tgSig,1)
        tgSig = tgSig(ann(1):ann(end-1)+subSeqLen+ovlpLen-1,channelInd);
        segGt = segGt(1:end-1);
    else
        tgSig = tgSig(ann(1):ann(end)+subSeqLen+ovlpLen-1,channelInd);
    end
    segGt = (segGt=='A');
end

end